import 'package:flutter/material.dart';

class AppTheme {

  static const primaryColor = Color(0xFF668D7B);
  static const secondaryColor = Color(0xFFBEDDCC);
  static const cardColor = Color(0xFF93B5A4);
  static const yellowColor = Color(0xFFF1CC83);
  static const redColor = Colors.red;
  static const brownColor = Color(0xFF6B6148);
  static const blackColor = Color(0xFF444444);
  static const textColor = Color(0xFFFFFFFF);

  static get theme => ThemeData(
    useMaterial3: true,
    colorScheme: ColorScheme.fromSeed(
      seedColor: primaryColor,
      surface: primaryColor,
    ),
    scaffoldBackgroundColor: primaryColor,
    textTheme: const TextTheme(
      bodySmall: bodySmall,
      bodyMedium: bodyMedium,
      bodyLarge: bodyLarge,
      titleLarge: titleLarge,
      headlineLarge: headlineLarge,
    ),
    elevatedButtonTheme:  ElevatedButtonThemeData(
      style: ButtonStyle(
        elevation: WidgetStateProperty.all(0),
        backgroundColor: WidgetStateProperty.all(textColor),
        padding: WidgetStateProperty.all(const EdgeInsets.symmetric(vertical: 0, horizontal: 4)),
        foregroundColor: WidgetStateProperty.all(primaryColor),
        textStyle: WidgetStateProperty.all(
          bodyMedium
        )
      ),
    ),
    outlinedButtonTheme: OutlinedButtonThemeData(
      style: ButtonStyle(
        elevation: WidgetStateProperty.all(0),
        backgroundColor: WidgetStateProperty.all(primaryColor),
        padding: WidgetStateProperty.all(const EdgeInsets.symmetric(vertical: 0, horizontal: 4)),
        foregroundColor: WidgetStateProperty.all(textColor),
        textStyle: WidgetStateProperty.all(
          bodyMedium
        )
      ),
    ),
    cardColor: cardColor,
  );

  static const bodySmall = TextStyle(
    color: textColor,
    fontSize: 14,
    fontWeight: FontWeight.w300,
    fontFamily: 'NotoSansArabic',
  );

  static const bodyMedium = TextStyle(
    color: textColor,
    fontSize: 16,
    fontWeight: FontWeight.w400,
    fontFamily: 'NotoSansArabic',
  );

  static const bodyLarge = TextStyle(
    color: textColor,
    fontSize: 18,
    fontWeight: FontWeight.w400,
    fontFamily: 'NotoSansArabic',
  );

  static const titleLarge = TextStyle(
    color: textColor,
    fontSize: 22,
    fontWeight: FontWeight.w600,
    fontFamily: 'NotoSansArabic',
  );

  static const headlineLarge = TextStyle(
    color: textColor,
    fontSize: 32,
    fontWeight: FontWeight.w700,
    fontFamily: 'NotoSansArabic',
  );
}